#!/bin/bash

# Install and setup httpd for ZendTo

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout Configure web server
shout
shout =================================================================
shout
pause

# Set up httpd
# For some strange reason, despite "hostname --help" listing the
# --fqdn option, it does not actually work. So we have to do it
# the hard way.
SERVERNAME="$( hostname --all-fqdns | grep -v localhost | head -1 | tr -d ' ' )"
if [ "x$SERVERNAME" = "x" ]; then
  SERVERNAME="$( hostname --all-fqdns | head -1 | tr -d ' ' )"
fi
CONFD=/etc/httpd/conf.d
HTTPDCONF=$CONFD/zendto.conf
HTTPDCONFS=$CONFD/zendto-ssl.conf
SSLCONF=$CONFD/ssl.conf
if [ -f $HTTPDCONF ]; then
  shout Looks like you already have a ZendTo website
  shout configured here. I will leave it alone.
else
  shout Creating http ZendTo website definition in
  shout '    '$HTTPDCONF
  shout "for ServerName $SERVERNAME"
  cat <<EOHTTPD > $HTTPDCONF
<VirtualHost *:80>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All

    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  #HTTPS# This will force http to automatically redirect to https.
  #HTTPS<IfModule mod_rewrite.c>
  #HTTPS  RewriteEngine On
  #HTTPS  RewriteCond %{HTTPS} off
  #HTTPS  RewriteCond %{REQUEST_URI} !^/\.well\-known/acme\-challenge/
  #HTTPS  RewriteRule (.*) https://%{HTTP_HOST}%{REQUEST_URI}
  #HTTPS</IfModule>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to get the WebDAV support working, so users
  # can easily upload files for the "library of files" feature.
  # I would recommend only doing this on the https site, and
  # not on this http one unless you cannot get a https certificate.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service apache2 restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>

</VirtualHost>
EOHTTPD
fi

if [ -f $HTTPDCONFS ]; then
  shout Looks like you already have a ZendTo HTTPS website
  shout configured here. I will leave it alone.
else
  shout Creating HTTPS ZendTo website definition in
  shout '    '$HTTPDCONFS
  shout "for ServerName $SERVERNAME"
  if [ -f "$SSLCONF" ]; then
    shout and commenting out default HTTPS site definition.
    sed -i '/^<VirtualHost /,/^<\/VirtualHost/ s/^/#ZendTo /' "$SSLCONF"
  fi

  cat <<EOHTTPDS > $HTTPDCONFS
<IfModule mod_ssl.c>

<VirtualHost *:443>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  # Enable SSL for this virtual host.
  SSLEngine on
  CustomLog logs/zendto_access_log combined
  ErrorLog  logs/zendto_error_log
  LogLevel warn

  # Where your SSL certificate and private key live.
  SSLCertificateFile    /etc/pki/tls/certs/zendto-selfsigned-cert.pem
  SSLCertificateKeyFile /etc/pki/tls/private/zendto-selfsigned-key.pem

  # ZendTo: Not needed for a self-signed certificate like the
  #         sample one above. But when you setup your proper
  #         certificate, put the Certificate Authority's certificate
  #         chain in here, and of course uncomment it!
  #SSLCertificateChainFile /etc/pki/tls/certs/server-chain.crt

  # Uncomment this when using client certificate authentication
  #SSLCACertificateFile /etc/pki/tls/certs/ca-bundle.crt

  # HSTS (mod_headers is required) (15768000 seconds = 6 months)
  Header always set Strict-Transport-Security "max-age=15768000"

  # CSP sets where different types of resource are allowed to come from
  # This is fairly strict except for images (img-src), as the service
  # logo may come from some web server of your own.
  # For further info read:
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/default-src
  Header set Content-Security-Policy "default-src 'none'; script-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.gstatic.cn https://www.recaptcha.net; connect-src 'self' 'unsafe-inline'; img-src data: * ; font-src 'self' 'unsafe-inline'  https://fonts.googleapis.com https://fonts.gstatic.com; style-src 'self' 'unsafe-inline' https://fonts.googleapis.com; frame-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.recaptcha.net"

  # SSL Protocol Adjustments for MSIE:
  # MSIE 7 and newer should be able to use keepalive
  BrowserMatch "MSIE [17-9]" ssl-unclean-shutdown

  <FilesMatch "\.php$">
    SSLOptions +StdEnvVars
  </FilesMatch>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All

    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to get the WebDAV support working, so users
  # can easily upload files for the "library of files" feature.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service httpd restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>
</VirtualHost>

EOHTTPDS

  if [ "$OSVER" -lt "6" ]; then
    cat <<EOHTTPS5 >> $HTTPDCONFS
# ZendTo: For CentOS / RedHat 5
# Get version numbers from "apache -v" and "openssl version"
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv2 -SSLv3
SSLCipherSuite          ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256:ECDHE-ECDSA-AES128-SHA:ECDHE-RSA-AES256-SHA384:ECDHE-RSA-AES128-SHA:ECDHE-ECDSA-AES256-SHA384:ECDHE-ECDSA-AES256-SHA:ECDHE-RSA-AES256-SHA:DHE-RSA-AES128-SHA256:DHE-RSA-AES128-SHA:DHE-RSA-AES256-SHA256:DHE-RSA-AES256-SHA:ECDHE-ECDSA-DES-CBC3-SHA:ECDHE-RSA-DES-CBC3-SHA:EDH-RSA-DES-CBC3-SHA:AES128-GCM-SHA256:AES256-GCM-SHA384:AES128-SHA256:AES256-SHA256:AES128-SHA:AES256-SHA:DES-CBC3-SHA:!DSS
SSLHonorCipherOrder     on

</IfModule>
EOHTTPS5
  elif [ "$OSVER" -eq "6" ]; then
    cat <<EOHTTPS6 >> $HTTPDCONFS
# ZendTo: For CentOS / RedHat 6
# Get version numbers from "apache -v" and "openssl version"
# apache 2.2.15 OpenSSL 1.0.1e
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv2 -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384
SSLHonorCipherOrder     off

</IfModule>
EOHTTPS6
  elif [ "$OSVER" -eq "7" ]; then
    cat <<EOHTTPS7 >> $HTTPDCONFS
# ZendTo: For CentOS / RedHat 7
# Get version numbers from "apache -v" and "openssl version"
# apache 2.4.6 OpenSSL 1.0.2k
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384
SSLHonorCipherOrder     off
SSLCompression          off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS7
  else
    cat <<EOHTTPS8 >> $HTTPDCONFS
# ZendTo: For Rocky / CentOS / RedHat 8
# Get version numbers from "apache -v" and "openssl version"
# apache 2.4.6 OpenSSL 1.0.2k
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384
SSLHonorCipherOrder     off
SSLSessionTickets       off
SSLCompression          off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS8
  fi

  chmod u=rw,go=r $HTTPDCONF $HTTPDCONFS
  shout Generating a self-signed SSL key and certificate for you.
  #
  # First we need to modify the openssl.cnf file.
  #
  # So make a safe copy of it
  OPENSSLCNF="$( mktemp )"
  cat /etc/pki/tls/openssl.cnf >> "$OPENSSLCNF"
  # Find the name of the x509 extensions section
  EXTENSIONS="$( grep '^x509_extensions' "$OPENSSLCNF" | head -1 | sed -e 's/x509_extensions[^=]*=[^a-zA-Z0-9]*\([a-zA-Z0-9_-]*\).*$/\1/' )"
  # Add the 2 extension lines s to the start of the right section
  shout "Including x509 certificate extensions"
  shout "extendedKeyUsage = serverAuth"
  shout "subjectAltName = DNS:$SERVERNAME"
  sed -i -e '/^\[ *'"$EXTENSIONS"' *\]/ a \\extendedKeyUsage = serverAuth\
subjectAltName = DNS:'"$SERVERNAME" "$OPENSSLCNF"
  # Now generate the cert including the new cnf file
  ( echo GB; echo Hampshire; echo Southampton; echo ZendTo User Site; echo; \
    echo $SERVERNAME; echo ) | \
  openssl req -x509 -sha256 -nodes -days 365 -newkey rsa:2048 -extensions "$EXTENSIONS" -config "$OPENSSLCNF" -keyout /etc/pki/tls/private/zendto-selfsigned-key.pem -out /etc/pki/tls/certs/zendto-selfsigned-cert.pem >/dev/null 2>&1
  # And ditch the temporary openssl.cnf file
  rm -f "$OPENSSLCNF"

  shout Enabling ssl, rewrite and headers modules.
  CONFFILE=/etc/httpd/conf/httpd.conf
  for M in ssl rewrite headers
  do
    if [ "$OSVER" -lt "7" ]; then
      # CentOS 5 and 6
      if grep -q '^ *LoadModule .*mod_'$M'.so' $CONFFILE /etc/httpd/conf.d/*.conf; then
        :
      else
        # We have to edit httpd.conf directly
        # Find the line number of the last "LoadModule" line
        LINE="$( grep -n '^LoadModule ' $CONFFILE | tail -1 | cut -d: -f1 )"
        if [ "x$LINE" = "x" ]; then
          shout Sorry, I cannot find the LoadModule lines in your $CONFFILE.
          shout You will need to check mod_$M is enabled yourself.
          shout It most likely is already.
        else
          sed -i $LINE' a \\LoadModule '$M'_module modules/mod_'$M'.so' $CONFFILE
        fi
      fi
    else
      # CentOS 7 has a conf.modules.d directory
      if grep -q '^ *LoadModule .*mod_'$M'.so' /etc/httpd/conf.modules.d/*.conf; then
        :
      else
        echo "LoadModule ${M}_module modules/mod_${M}.so" >> /etc/httpd/conf.modules.d/zendto.conf
      fi
    fi
  done

  shout
  shout 'The http and https versions of your new ZendTo site'
  shout 'will be at the addresses'
  shout '    'http://$SERVERNAME'/'
  shout and
  shout '    'https://$SERVERNAME'/'
  shout
  shout '*** Please note these down as you will need them in a minute. ***'
  shout
  shout I can configure the http site to automatically forward
  shout all connections to the https site.
  shout But because I have just used a self-signed certificate,
  shout your web browser will complain very loudly about the
  shout security of the https site the first time you visit it.
  shout
  shout 'You should normally say "yes" to this question.'
  shout 'But you might have a good reason to say "no", such as you'
  shout are just trying it out internally and are not bothered
  shout about security right now on this server.
  shout
  shout Would you like me to automatically forward http
  if yesno "connections straight to the https site" "y"; then
    sed -i 's/#HTTPS//g;' $HTTPDCONF
    shout Connections to http will go straight to https.
  fi
fi

# Select the right time zone as we need it for php.ini
shout
shout I need to know which timezone you are in.
shout
# If they are running systemd, they'll have timedatectl so use it
MYTZ="$( timedatectl status 2>/dev/null |
         grep -i 'time *zone:' |
         sed -e 's/^[^:]*: *//; s/ *(.*$//;' )"
# If that failed (don't have systemd) then see if /etc/localtime points to it
if [ "x$MYTZ" = "x" -a -L /etc/localtime ]; then
  # Timezone names start with a capital letter or a number,
  # the parent directory names don't.
  MYTZ="$( readlink /etc/localtime | perl -p -e 's/^.*?\/([A-Z0-9].*$)/$1/' )"
fi
MYTZ="$(prompt "Time zone" "America/New_York" "$MYTZ")"
shout "Thank you, got your time zone as $MYTZ"
#shout "Setting your /etc/localtime to $MYTZ"
#[ ! -L /etc/localtime ] && ln -sf /usr/share/zoneinfo/"$MYTZ" /etc/localtime
#pause

# Setup php.ini and shout
shout Configuring PHP
pause

# Set configuration in php.ini or php.d/99-zendto.ini
if [ -d /etc/php.d ]; then
  F=/etc/php.d/99-zendto.ini
else
  F=/etc/php.ini
fi
shout Patching $F
if [ -f "$F" ]; then
  cp -f "$F" "$F".zendto
else
  touch "$F"
fi
setphpini $F date.timezone "$MYTZ"
setphpini $F max_execution_time 28800
setphpini $F max_input_time 28800
setphpini $F memory_limit 300M
setphpini $F error_reporting 'E_ALL & ~E_DEPRECATED & ~E_STRICT & ~E_NOTICE'
setphpini $F display_errors Off
setphpini $F display_startup_errors Off
setphpini $F log_errors On
setphpini $F post_max_size 50000M
setphpini $F file_uploads On
setphpini $F upload_tmp_dir /var/zendto/incoming
setphpini $F upload_max_filesize 50G
setphpini $F max_file_uploads 200
setphpini $F browscap /opt/zendto/lib/browscap.ini

shout
shout Your web server and PHP ini files should now be
shout configured to run ZendTo on this server.
shout As mentioned above, you will need to setup your
shout own https SSL certificate and keys.
shout To find the basic settings for ZendTo, look in
shout '    '$HTTPDCONF
shout '    '$HTTPDCONFS
shout

exit 0

