#!/usr/bin/env bash

# Install ZendTo itself.

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout 'Install ZendTo itself and configure email sending'
shout
shout =================================================================
shout
pause

shout Installing usage statistics graphing package, gettext translation tool
shout and jq utility
pkg install --yes rrdtool gettext jq

# Do we want to use the repo?
USEREPO=n

shout ' '
mkdir -p "$SRCSTORE" && cd "$SRCSTORE" || exit 1

shout And now fetch and unpack ZendTo
VERSION="$( curl --silent https://zend.to/files/ZendTo-Version-Beta )"
shout "According to the zend.to website, the latest version is $VERSION."
VERSION="$( prompt "Version of ZendTo to install" "5.19-1" "$VERSION")"
curl -O https://zend.to/files/ZendTo-"$VERSION".tgz || {
  shout "Failed to find ZendTo tgz for version ${VERSION}."
  shout "Exiting..."
  exit 1
}

# Might not exist yet on FreeBSD
mkdir -p /opt

if [ -e /opt/zendto ]; then
  if [ -L /opt/zendto ]; then
    OLDVER="$( readlink -fn /opt/zendto )"
    shout Unhooking your old version at $OLDVER
    rm -f /opt/zendto
  else
    OLDVER=/opt/zendto
    RENAMETO="/opt/ZendTo-$( date +%Y%m%d )"
    shout Renaming your old version at $OLDVER to $OLDVER.$RENAMETO
    mv -f $OLDVER $OLDVER.$RENAMETO
  fi
fi

shout "About to install ZendTo version $VERSION into /opt/ZendTo-$VERSION"
pause
tar -x -z -C /opt -f ZendTo-"$VERSION".tgz

shout "and link it to /opt/zendto"
ln -nsf ZendTo-"$VERSION" /opt/zendto

shout "and construct /var/zendto tree"
# Work out what the Apache username and group are
WWWUSER="$( grep -ri --no-filename '^User ' /usr/local/etc/apache24 2>/dev/null | head -1 | sed -e 's/#.*$//' | awk '{ print $2 }' )"
WWWGROUP="$( grep -ri --no-filename '^Group ' /usr/local/etc/apache24 2>/dev/null | head -1 | sed -e 's/#.*$//' | awk '{ print $2 }' )"
if [ "x$WWWUSER" = "x" ]; then
  WWWUSER='www'
fi
if [ "x$WWWGROUP" = "x" ]; then
  WWWGROUP='www'
fi

# Construct /var/zendto and others.
# Cannot pre-determine the ownership as we can only read this from
# httpd.conf, when we are on the target system and installing ourselves.
shout Constructing the /var/zendto tree
shout and setting its ownership and permissions
mkdir -p /var/zendto
chown root:"$WWWGROUP" /var/zendto
chgrp "$WWWGROUP" /var/zendto
chmod 0775 /var/zendto
for F in incoming dropoffs rrd library cache templates_c
do
  if [ \! -d /var/zendto/$F/ ]; then
    mkdir -p /var/zendto/$F
    chown "$WWWUSER":"$WWWGROUP" /var/zendto/$F
    chmod 0755 /var/zendto/$F
  fi
done
for F in cache templates_c
do
  :> /var/zendto/$F/This.Dir.Must.Be.Writeable.By.Apache
  chown "$WWWUSER":"$WWWGROUP" /var/zendto/$F/This.Dir.Must.Be.Writeable.By.Apache
  chmod u=rw,g=rw,o=r /var/zendto/$F/This.Dir.Must.Be.Writeable.By.Apache
done
cp /opt/zendto/www/images/notfound.png /var/zendto/rrd/notfound.png
chmod a+r /var/zendto/rrd/notfound.png
# Clean the caches in case Smarty has been upgraded
rm -f /var/zendto/cache/*php >/dev/null 2>&1
rm -f /var/zendto/templates_c/*php >/dev/null 2>&1
# Remove obsolete caches in case they are still there
rm -rf /opt/zendto/cache >/dev/null 2>&1
rm -rf /opt/zendto/templates_c >/dev/null 2>&1
shout And /var/log/zendto for the log files
mkdir -p /var/log/zendto
chgrp "$WWWGROUP" /var/log/zendto
chmod 0775 /var/log/zendto
touch /var/log/zendto/zendto.log
chown "$WWWUSER":"$WWWGROUP" /var/log/zendto/zendto.log
chmod u=rw,g=rw,o=r /var/log/zendto/zendto.log

for F in zendto.conf preferences.php internaldomains.conf
do
  chgrp "$WWWGROUP" /opt/zendto/config/$F
done
chgrp "$WWWGROUP" /opt/zendto/config/locale/*_*/LC_MESSAGES/zendto.{po,mo}* 2>/dev/null

# Install cshrc, profile, cron and newsyslog jobs
if ! grep -q '^setenv  *ZENDTOPREFS' /etc/csh.cshrc; then
  shout Setting the ZENDTOPREFS environment variable for csh system-wide
  echo "# Added for ZendTo" >> /etc/csh.cshrc
  echo "setenv ZENDTOPREFS /opt/zendto/config/preferences.php" >> /etc/csh.cshrc
fi
if ! grep -q '^export  *ZENDTOPREFS' /etc/profile; then
  shout Setting the ZENDTOPREFS environment variable for sh and bash system-wide
  echo "# Added for ZendTo" >> /etc/profile
  echo "export ZENDTOPREFS=/opt/zendto/config/preferences.php" >> /etc/profile
fi
if [ \! -f /etc/cron.d/zendto ]; then
  shout Adding the cron jobs to do nightly housekeeping
  cat > /etc/cron.d/zendto <<'EOCRON'
# Cron jobs for ZendTo housekeeping and statistics collection
SHELL=/usr/local/bin/bash
5 0 * * * root php /opt/zendto/sbin/cleanup.php /opt/zendto/config/preferences.php >/dev/null 2>&1
5 */4 * * * root find -H /var/zendto/incoming -type f -mmin +240 -delete >/dev/null 2>&1
1 1 * * * root php /opt/zendto/sbin/rrdInit.php /opt/zendto/config/preferences.php 2>&1 | /bin/grep -iv 'illegal attempt to update using time'
3 3 * * * root php /opt/zendto/sbin/rrdUpdate.php /opt/zendto/config/preferences.php 2>&1 | grep -v '^[0-9]*x[0-9]*$'
EOCRON
fi
if [ \! -f /etc/newsyslog.conf.d/zendto ]; then
  shout Setting up the logs to auto-rotate themselves
  cat > /etc/newsyslog.conf.d/zendto <<EONEWSYSLOG
# Rotate the ZendTo logs automatically at 11pm every Sunday
/var/log/zendto/zendto.log $WWWUSER:$WWWGROUP 664 10 * \$W0D23 CNpZ
/var/zendto/zendto.log     $WWWUSER:$WWWGROUP 664 10 1 \$W0D23 CNpZ
EONEWSYSLOG
fi

shout
shout And set up graphing data
php /opt/zendto/sbin/rrdUpdate.php /opt/zendto/config/preferences.php | grep -v '^[0-9]*x[0-9]*$'

if [ -x /opt/zendto/bin/makelanguages ]; then
  shout
  shout 'And update and compile translations'
  /opt/zendto/bin/makelanguages
  chmod -R go+rX /opt/zendto/config/locale/*_*
fi

# Set up the SMTP settings in preferences.php
configurePHPMailer
# Set up the internaldomains.conf file
configureInternalDomains
# Set up their zendto.conf file
configureZendToConf

# If we are using mod_rewrite to hide PHP extensions, tell ZendTo
if grep -q '^ *RewriteRule.*1\.php' /usr/local/etc/apache24/Includes/zendto-ssl.conf 2>/dev/null; then
  shout 'Enabled PHP hiding'
  setPrefphp hidePHP notquoted "TRUE"
fi

# Set the cookieSecret in preferences.php
shout "Setting 'cookieSecret' in ZendTo preferences.php"
shout "If this takes a long time, it needs more entropy (randomness)."
shout "So wiggle your mouse around, login separately to the ZendTo server"
shout "and run commands, type stuff, make it do almost anything. It will"
shout "get there eventually."
COOKIESECRET="$( /opt/zendto/sbin/genCookieSecret.php | grep "'cookieSecret' *=" | awk '{ print $NF }' | tr -d "'" )"
setPrefphp cookieSecret quoted "$COOKIESECRET"

# Set the http/https bit of the serverRoot setting in preferences.php
# First find the site config file, then work out if we're sure that
# we wrote it, and the RewriteRule is not commented out.
# If so, change the http to https in 'serverRoot'.
SERVERNAME="$( hostname -f )"
HTTPDCONF=/usr/local/etc/apache24/Includes/zendto.conf
if [ -f "$HTTPDCONF" ]; then
  # The site is there
  if grep -q '^ *RewriteRule.*https:' "$HTTPDCONF"; then
    # The RewriteRule is in place, so we're forwarding to https
    SERVERROOT="https://$SERVERNAME/"
    DEFROOT="https://zendto.soton.ac.uk/"
  else
    SERVERROOT="http://$SERVERNAME/"
    DEFROOT="http://zendto.soton.ac.uk/"
  fi
  shout
  shout "The address of your new ZendTo site MUST end with a '/'"
  SERVERROOT="$( prompt "What is the address of your new ZendTo site" "$DEFROOT" "$SERVERROOT")"
  setPrefphp serverRoot quoted "$SERVERROOT"
  shout "I have set your 'serverRoot' value in preferences.php to"
  shout "  'serverRoot' => '$SERVERROOT'"
  shout 'Be sure to check this is correct, or ZendTo will fail in'
  shout 'strange ways.'
  pause 10
fi

shout
shout
shout 'Please ensure your /opt/zendto/config/* files are up to date.'
shout
shout 'To help you, there is a tool for upgrading the preferences.php'
shout 'and zendto.conf files.'
shout 'Simply run'
shout '    /opt/zendto/bin/upgrade'
shout 'and it should do the job.'
shout
shout 'To edit any text in the web interface, or add/edit a language,'
shout 'see'
shout '    https://zend.to/translators'
shout
pause 20

shout
shout ZendTo itself has been installed.
shout

exit 0
