#!/bin/bash

# Rebuild PHP with SQLite+IMAP (if not already included) and big-uploads support.

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout 'Install PHP if not already installed, preferably PHP 7.2, but'
shout 'at least 7.0.'
shout 'If not PHP 7.2, build and install the "sodium" PHP extension.'
shout
shout =================================================================
shout
pause

# Is PHP installed? If so, what version?
PHPVER='none'
PHPMAJOR=0
PHPPREFIX=''

# Find the php binary first. Let's hope 'which' is installed.
PHPBIN="$( which php 2>/dev/null )"
if [ "x$PHPBIN" != "x" ]; then
  PHPVER="$( $PHPBIN -i |
             grep '^PHP *Version' |
             head -1 |
             awk '{ print $NF }' )"
  shout Found PHP version "$PHPVER"
  # This gives us php7
  PHPPREFIX="$( rpm -qf /usr/bin/php | sed -e 's/-.*$//' )"
  # Reduce it to 72
  PHPVER="$( echo "$PHPVER" | sed -e 's/^\([^.]*\)\.\([^.]*\)\..*/\1\2/' )"
  # And 7
  PHPMAJOR="$( echo "$PHPVER" | cut -d. -f1 )"
  # Didn't work? Version=0 will force new installation
  if [ "x$PHPMAJOR" = "x" ]; then
    PHPMAJOR=0
    PHPVER=0
  fi
fi

if [ "$OS" = "sles" -a "$OSVER" -ge 12 ]; then
  shout Good, you are on SLES 12 or later, so PHP 7 is available ready-built.
elif [ "$OS" = "opensuse" -a "$OSVER" -ge 15 ]; then
  shout Good, you are on openSUSE 15 or later, so PHP 7 is available ready-built.
else
  shout I am relying on PHP7 being available, which it might not be
  shout on your version of "$OS".
  shout This next step may well fail. I need PHP7.
  pause
fi

# Find the most recent version of PHP we can get
# This gives us php7
PHPPREFIXLATEST="$( zypper search --type=package 'php>=7' | grep '\sphp[0-9]*\s' | sort -V -k +6 | tail -1 | awk '{ print $2 }' )"
# This gives us 70 or 72
PHPVERLATEST="$( zypper search --type=package 'php>=7' | grep '\sphp[0-9]*\s' | sort -V -k +6 | tail -1 | awk '{ print $6 }' | cut -d. -f1,2 | tr -d . )"

# We either have no PHP, or PHP 7 or newer, or an old useless PHP
if (( PHPMAJOR == 0 || PHPMAJOR >= 7 )); then
  if (( PHPMAJOR == 0 )); then
    shout Okay, no PHP installed. I am going to install "$PHPPREFIXLATEST".
  else
    shout Currently installed PHP is at least version 7, so I will
    shout just add to your existing RPM-based PHP installation.
  fi
  PHPPREFIXLATEST="$(prompt "Prefix of PHP package names" "php7" "$PHPPREFIXLATEST")"
  shout Install the Apache PHP module and any missing PHP packages that we will need.
  zypper --non-interactive install sqlite3 apache2-mod_"$PHPPREFIXLATEST" "$PHPPREFIXLATEST" "$PHPPREFIXLATEST"-{pdo,mysql,sqlite,ldap,intl,gd,enchant,json,mbstring,gettext}
  shout
  shout Some SLES and openSUSE releases are missing the "$PHPPREFIXLATEST"-imap package
  shout for some unknown reason. So the next command might fail.
  pause
  zypper --non-interactive install "$PHPPREFIXLATEST"-imap
  PHPVER="$PHPVERLATEST"
  PHPPREFIX="$PHPPREFIXLATEST"
else
  shout 'Sorry, I do not know how to replace PHP5 with PHP7.'
fi

# How do we now install sodium?
if (( PHPVER >= 72 )); then
  shout 'You are running PHP 7.2 or later, so the "sodium" extension is included'
  zypper --non-interactive install "$PHPPREFIX"-sodium
else
  shout 'We need to use PECL to install the sodium extension.'
  shout 'For that we will need some development tools.'
  pause
  if [ "$OS" = "sles" ]; then
    shout 'Registering to use SLES "SDK Module"'
    shout '(This will take a few seconds, be patient...)'
    if [ "$OSVER" = "12" ]; then
      SLESVER=12.3
    else
      SLESVER="$OSVER"
    fi
    SUSEConnect -p sle-sdk/"$SLESVER"/x86_64
    zypper --non-interactive install pattern:Basis-Devel
  else
    zypper --non-interactive install --type pattern devel_C_C++
  fi
  zypper --non-interactive install "$PHPPREFIX"-devel
  mkdir -p "$SRCSTORE" && cd "$SRCSTORE" || exit 1
  rm -f LATEST.tar.gz
  shout 'Downloading libsodium...'
  curl -O https://download.libsodium.org/libsodium/releases/LATEST.tar.gz
  tar xzf LATEST.tar.gz
  cd libsodium-stable
  shout 'Now to build libsodium.'
  pause
  ./configure
  shout 'Compiling...'
  pause
  make
  shout 'Testing...'
  pause
  make check
  shout 'Installing...'
  pause
  make install
  shout 'And now use PECL to build and install the PHP extension'
  pecl install libsodium
  shout 'and enable it in PHP'
  cat > /etc/"$PHPPREFIX"/conf.d/sodium.ini <<EOINI
; ZendTo Enable sodium extension module
extension=sodium.so
EOINI
fi
  
shout 'You should now have all the PHP packages installed.'

exit 0
