#!/bin/bash

# Install and setup httpd for ZendTo

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout Configure web server
shout
shout =================================================================
shout
pause

# Set up httpd
SERVERNAME="$( hostname --fqdn )"
CONFD=/etc/apache2/vhosts.d
HTTPDCONF=$CONFD/zendto.conf
HTTPDCONFS=$CONFD/zendto-ssl.conf
SSLCONF=/etc/apache2/ssl-global.conf
if [ -f $HTTPDCONF ]; then
  shout Looks like you already have a ZendTo website
  shout configured here. I will leave it alone.
else
  shout Creating http ZendTo website definition in
  shout $HTTPDCONF
  shout "for ServerName $SERVERNAME"
  cat <<EOHTTPD > $HTTPDCONF
<VirtualHost *:80>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All

    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  #HTTPS# This will force http to automatically redirect to https.
  #HTTPS<IfModule mod_rewrite.c>
  #HTTPS  RewriteEngine On
  #HTTPS  RewriteCond %{HTTPS} off
  #HTTPS  RewriteRule (.*) https://%{HTTP_HOST}%{REQUEST_URI}
  #HTTPS</IfModule>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to get the WebDAV support working, so users
  # can easily upload files for the "library of files" feature.
  # I would recommend only doing this on the https site, and
  # not on this http one unless you cannot get a https certificate.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service apache2 restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>

</VirtualHost>
EOHTTPD
fi

if [ -f $HTTPDCONFS ]; then
  shout Looks like you already have a ZendTo HTTPS website
  shout configured here. I will leave it alone.
else
  shout Creating HTTPS ZendTo website definition in
  shout '    '$HTTPDCONFS
  shout "for ServerName $SERVERNAME"
  if [ -f "$SSLCONF" ]; then
    shout and commenting out any default HTTPS site definition.
    sed -i '/^<VirtualHost /,/^<\/VirtualHost/ s/^/#ZendTo /' "$SSLCONF"
  fi

  cat <<EOHTTPDS > $HTTPDCONFS
<IfModule mod_ssl.c>

<VirtualHost *:443>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  # Enable SSL for this virtual host.
  SSLEngine on
  #CustomLog logs/zendto_access_log combined
  #ErrorLog  logs/zendto_error_log
  LogLevel warn

  # Where your SSL certificate and private key live.
  SSLCertificateFile    /etc/apache2/ssl.crt/zendto-selfsigned-cert.pem
  SSLCertificateKeyFile /etc/apache2/ssl.key/zendto-selfsigned-key.pem

  # ZendTo: Not needed for a self-signed certificate like the
  #         sample one above. But when you setup your proper
  #         certificate, put the Certificate Authority's certificate
  #         chain in here, and of course uncomment it!
  #SSLCertificateChainFile /etc/apache2/ssl.crt/server-chain.crt

  # Uncomment this when using client certificate authentication
  #SSLCACertificateFile /etc/pki/tls/certs/ca-bundle.crt

  # HSTS (mod_headers is required) (15768000 seconds = 6 months)
  Header always set Strict-Transport-Security "max-age=15768000"

  # CSP sets where different types of resource are allowed to come from
  # This is fairly strict except for images (img-src), as the service
  # logo may come from some web server of your own.
  # For further info read:
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/default-src
  Header set Content-Security-Policy "default-src 'none'; script-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.gstatic.cn https://www.recaptcha.net; connect-src 'self' 'unsafe-inline'; img-src data: * ; font-src 'self' 'unsafe-inline'  https://fonts.googleapis.com https://fonts.gstatic.com; style-src 'self' 'unsafe-inline' https://fonts.googleapis.com; frame-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.recaptcha.net"

  # SSL Protocol Adjustments for MSIE:
  # MSIE 7 and newer should be able to use keepalive
  BrowserMatch "MSIE [17-9]" ssl-unclean-shutdown

  <FilesMatch "\.php$">
    SSLOptions +StdEnvVars
  </FilesMatch>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All

    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to get the WebDAV support working, so users
  # can easily upload files for the "library of files" feature.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service apache2 restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>
</VirtualHost>

EOHTTPDS

  if [ "$OS" = "sles" ]; then
    cat <<EOHTTPS12 >> $HTTPDCONFS
# ZendTo: For SuSE 12+
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
SSLHonorCipherOrder     on
SSLCompression          off
SSLSessionTickets       off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS12
  elif [ "$OS" = "opensuse" ]; then
    cat <<EOHTTPS15 >> $HTTPDCONFS
# ZendTo: For openSUSE Leap 15
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
SSLHonorCipherOrder     on
SSLCompression          off
SSLSessionTickets       off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS15
  else
    shout 'Not SLES or openSUSE? Not possible!'
  fi


  chmod u=rw,go=r $HTTPDCONF $HTTPDCONFS
  shout Generating a self-signed SSL key and certificate for you.

  #
  # First we need to modify the openssl.cnf file.
  #
  # So make a safe copy of it
  OPENSSLCNF="$( mktemp )"
  cat /etc/ssl/openssl.cnf >> "$OPENSSLCNF"
  # Find the name of the x509 extensions section
  EXTENSIONS="$( grep '^x509_extensions' "$OPENSSLCNF" | head -1 | sed -e 's/x509_extensions[^=]*=[^a-zA-Z0-9]*\([a-zA-Z0-9_-]*\).*$/\1/' )"
  # Add the 2 extension lines s to the start of the right section
  shout "Including x509 certificate extensions"
  shout "extendedKeyUsage = serverAuth"
  shout "subjectAltName = DNS:$SERVERNAME"
  sed -i -e '/^\[ *'"$EXTENSIONS"' *\]/ a \\extendedKeyUsage = serverAuth\
subjectAltName = DNS:'"$SERVERNAME" "$OPENSSLCNF"
  # Now generate the cert including the new cnf file
  ( echo GB; echo Hampshire; echo Southampton; echo ZendTo User Site; echo; \
    echo $SERVERNAME; echo ) | \
  openssl req -x509 -sha256 -nodes -days 365 -newkey rsa:2048 -extensions "$EXTENSIONS" -config "$OPENSSLCNF" -keyout /etc/apache2/ssl.key/zendto-selfsigned-key.pem -out /etc/apache2/ssl.crt/zendto-selfsigned-cert.pem >/dev/null 2>&1
  # And ditch the temporary openssl.cnf file
  rm -f "$OPENSSLCNF"

  shout Enabling php7, ssl, rewrite and headers modules.
  CONFFILE=/etc/httpd/conf/httpd.conf
  for M in php7 ssl rewrite headers
  do
    a2enmod "$M"
  done

  shout Adding SSL to the Apache startup flags if not there...
  EDITAPACHE2=no
  SYSCONF="/etc/sysconfig/apache2"
  FLAGS="$( grep '^APACHE_SERVER_FLAGS=' "$SYSCONF" | sed -e 's/^[^=]*="\([^"]*\)".*$/\1/' )"
  if [[ $FLAGS =~ NOSSL ]]; then
    shout Removing NOSSL flag
    FLAGS="$( echo "$FLAGS" | sed -e 's/NOSSL//g' )"
    EDITAPACHE2=yes
  fi
  if [[ $FLAGS =~ SSL ]]; then
    :
  else
    shout Adding SSL flag
    FLAGS="${FLAGS} SSL"
    EDITAPACHE2=yes
  fi
  if [ "x$EDITAPACHE2" = "xyes" ]; then
    # Clean up FLAGS string: remove multiple spaces, leading & trailing spaces
    FLAGS="$( echo "$FLAGS" | tr -s ' ' | sed -e 's/^ *//g; s/ *$//g;' )"
    sed -i -e 's/^APACHE_SERVER_FLAGS=.*$/APACHE_SERVER_FLAGS="'"$FLAGS"'"/' "$SYSCONF"
    shout Edited "$SYSCONF"
  else
    shout No changes needed
  fi

  shout
  shout 'The http and https versions of your new ZendTo site'
  shout 'will be at the addresses'
  shout '    'http://$SERVERNAME'/'
  shout and
  shout '    'https://$SERVERNAME'/'
  shout
  shout '*** Please note these down as you will need them in a minute. ***'
  shout
  shout I can configure the http site to automatically forward
  shout all connections to the https site.
  shout But because I have just used a self-signed certificate,
  shout your web browser will complain very loudly about the
  shout security of the https site the first time you visit it.
  shout
  shout 'You should normally say "yes" to this question.'
  shout 'But you might have a good reason to say "no", such as you'
  shout are just trying it out internally and are not bothered
  shout about security right now on this server.
  shout
  shout Would you like me to automatically forward http
  if yesno "connections straight to the https site" "y"; then
    sed -i 's/#HTTPS//g;' $HTTPDCONF
    shout Connections to http will go straight to https.
  fi
fi

# Select the right time zone as we need it for php.ini
shout
shout I need to know which timezone you are in.
shout
# If they are running systemd, they'll have timedatectl so use it
MYTZ="$( timedatectl status 2>/dev/null |
         grep -i 'time *zone' |
         sed -e 's/^[^:]*: *//; s/ *(.*$//;' )"
# If that failed (don't have systemd) then see if /etc/localtime points to it
if [ "x$MYTZ" = "x" -a -L /etc/localtime ]; then
  # Timezone names start with a capital letter or a number,
  # the parent directory names don't.
  MYTZ="$( readlink /etc/localtime | perl -p -e 's/^.*?\/([A-Z0-9].*$)/$1/' )"
fi
## else ask them for it
#if [ "x$MYTZ" = "x" ]; then
#  MYTZ="$(tzselect | tail -1)"
#  echo "$MYTZ"
#fi
#echo
MYTZ="$(prompt "Time zone" "America/New_York" "$MYTZ")"
shout "Thank you, got your time zone as $MYTZ"
#shout "Setting your /etc/localtime to $MYTZ"
#[ ! -L /etc/localtime ] && ln -sf /usr/share/zoneinfo/"$MYTZ" /etc/localtime
#pause

# Setup php.ini and /etc/php.d/apc.ini or apcu.ini
shout
shout Configuring PHP
pause

# Set configuration in php.ini
F=/etc/php7/apache2/php.ini
shout Patching $F
cp -f $F $F.zendto
setphpini $F date.timezone "$MYTZ"
setphpini $F max_execution_time 28800
setphpini $F max_input_time 28800
setphpini $F memory_limit 300M
setphpini $F error_reporting 'E_ALL & ~E_DEPRECATED & ~E_STRICT & ~E_NOTICE'
setphpini $F display_errors Off
setphpini $F display_startup_errors Off
setphpini $F log_errors On
setphpini $F post_max_size 50000M
setphpini $F file_uploads On
setphpini $F upload_tmp_dir /var/zendto/incoming
setphpini $F upload_max_filesize 50G
setphpini $F max_file_uploads 200
setphpini $F browscap /opt/zendto/lib/browscap.ini

shout
shout Your web server and PHP ini files should now be
shout configured to run ZendTo on this server.
shout As mentioned above, you will need to setup your
shout own https SSL certificate and keys.
shout To find the basic settings for ZendTo, look in
shout '    '$HTTPDCONF
shout '    '$HTTPDCONFS
shout

exit 0

