#!/bin/bash

# Install ZendTo itself.

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout 'Install ZendTo itself and configure email sending'
shout
shout =================================================================
shout
pause

shout Installing usage statistics graphing package, nc and gettext
zypper --non-interactive install rrdtool netcat-openbsd gettext-tools

# Do we want to use the repo?
if [ "x$USEREPO" = "x" ]; then
  # This setting will be replaced with n if a beta installer
  USEREPO=n
fi

shout ' '
mkdir -p "$SRCSTORE" && cd "$SRCSTORE" || exit 1
REPOFAILED=yes
if [ "x$USEREPO" = "xy" ]; then
  if zypper repos ZendTo >/dev/null 2>&1; then
    shout "The ZendTo repository already exists."
    REPOFAILED=no
  else
    shout "Trying to use the ZendTo yum repository."
    # Install the zendto-repo RPM in order to get the zendto.repo file
    rpm --import https://zend.to/files/zendto.gpg.asc
    zypper --non-interactive --quiet --no-color --gpg-auto-import-keys install https://zend.to/files/zendto-repo.rpm && \
    REPO="$( rpm -ql zendto-repo | fgrep zendto.repo )" && \
    # Add the repo now where we now the zendto.repo file is
    zypper --non-interactive --quiet --no-color addrepo -f -G -t rpm-md "$REPO" && \
    # Remove the zendto-repo RPM again, now we're done with it
    rpm -e zendto-repo && \
    REPOFAILED=no
    # Clean up if that dir is empty
    rmdir /etc/yum.repos.d 2>/dev/null
  fi
else
  # Fake repo failure, as we didn't want to use it anyway
  REPOFAILED=yes
fi

if [ "x$REPOFAILED" = "xno" ]; then
  shout "Now to install ZendTo itself from the yum repository."
  shout "Drumroll please..."
  pause
  if rpm --quiet -q zendto; then
    shout Upgrading ZendTo
    zypper --non-interactive update zendto
  else
    shout Installing ZendTo
    zypper --non-interactive install zendto
  fi
else
  shout "Failed to find ZendTo yum repository rpm."
  shout "So we will find and install the latest version without it."
  VERSION="$( curl --silent https://zend.to/files/ZendTo-Version-Beta )"
  shout "According to the zend.to website, the latest version is $VERSION."
  VERSION="$( prompt "Version of ZendTo to install" "4.20-1" "$VERSION")"
  curl -O https://zend.to/files/zendto-"$VERSION".noarch.rpm || {
    shout "Failed to find ZendTo rpm for version ${VERSION}."
    shout "Exiting..."
    exit 1
  }
  INSTALLEDVER="$( rpm -q --queryformat='%{VERSION}-%{RELEASE}' zendto | grep '^[0-9]' )"
  if [ "x$INSTALLEDVER" = "x$VERSION" ]; then
    shout "Version $INSTALLEDVER is already installed. Skipping."
  else
    shout "About to install ZendTo version $VERSION"
    shout "Slightly shorter and quieter drumroll please..."
    pause
    rpm -Uvh "zendto-${VERSION}.noarch.rpm" || {
      shout Failed to install ZendTo rpm.
      shout Exiting...
      exit 1
    }
  fi
fi

shout
shout And set up graphing data
php /opt/zendto/sbin/rrdUpdate.php /opt/zendto/config/preferences.php | grep -v '^[0-9]*x[0-9]*$'

if [ -x /opt/zendto/bin/makelanguages ]; then
  shout
  shout 'And update and compile translations'
  /opt/zendto/bin/makelanguages
fi

# Set up the SMTP settings in preferences.php
configurePHPMailer
# Set up the internaldomains.conf file
configureInternalDomains
# Set up their zendto.conf file
configureZendToConf

# If we are using mod_rewrite to hide PHP extensions, tell ZendTo
if grep -q '^ *RewriteRule.*1\.php' /etc/apache2/vhosts.d/zendto-ssl.conf 2>/dev/null 2>/dev/null; then
  shout 'Enabled PHP hiding'
  setPrefphp hidePHP notquoted "TRUE"
fi

# Set the cookieSecret in preferences.php
shout "Setting 'cookieSecret' in ZendTo preferences.php"
shout "If this takes a long time, it needs more entropy (randomness)."
shout "So wiggle your mouse around, login separately to the ZendTo server"
shout "and run commands, type stuff, make it do almost anything. It will"
shout "get there eventually."
COOKIESECRET="$( /opt/zendto/sbin/genCookieSecret.php | grep "'cookieSecret' *=" | awk '{ print $NF }' | tr -d "'" )"
setPrefphp cookieSecret quoted "$COOKIESECRET"

# Set the http/https bit of the serverRoot setting in preferences.php
# First find the site config file, then work out if we're sure that
# we wrote it, and the RewriteRule is not commented out.
# If so, change the http to https in 'serverRoot'.
#
# For some strange reason, despite "hostname --help" listing the
# --fqdn option, it does not actually work. So we have to do it
# the hard way.
SERVERNAME="$( hostname --fqdn )"
HTTPDCONF=/etc/apache2/vhosts.d/zendto.conf
if [ -f "$HTTPDCONF" ]; then
  # The site is there
  if grep -q '^ *RewriteRule.*https:' "$HTTPDCONF"; then
    # The RewriteRule is in place, so we're forwarding to https
    SERVERROOT="https://$SERVERNAME/"
    DEFROOT="https://zendto.soton.ac.uk/"
  else
    SERVERROOT="http://$SERVERNAME/"
    DEFROOT="http://zendto.soton.ac.uk/"
  fi
  shout
  shout "The address of your new ZendTo site MUST end with a '/'"
  SERVERROOT="$( prompt "What is the address of your new ZendTo site" "$DEFROOT" "$SERVERROOT")"
  setPrefphp serverRoot quoted "$SERVERROOT"
  shout "I have set your 'serverRoot' value in preferences.php to"
  shout "  'serverRoot' => '$SERVERROOT'"
  shout 'Be sure to check this is correct, or ZendTo will fail in'
  shout 'strange ways.'
  pause 10
fi

shout
shout
shout 'Please ensure your /opt/zendto/config/* files are up to date.'
shout
shout 'To help you, there is a tool for upgrading the preferences.php'
shout 'and zendto.conf files.'
shout 'Simply run'
shout '    /opt/zendto/bin/upgrade'
shout
shout 'To edit any text in the web interface, or add/edit a language,'
shout 'see'
shout '    https://zend.to/translators'
shout
pause 20

shout
shout ZendTo itself has been installed.
shout

exit 0
