#!/bin/bash

# Install PHP and its modules.

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout 'Install at least PHP 7.2, and all the modules I need.'
shout
shout =================================================================
shout
pause

# If they are running Ubuntu before 18, they need Ondrej Suri's repo.
if [ "$OS" = "ubuntu" -a "$OSVER" -lt "18" ]; then
  shout Installing Ondrej Suri\'s APT repo to get PHP.
  LC_ALL=C.UTF-8 add-apt-repository --yes ppa:ondrej/php
  if [ "$?" != "0" ]; then
    shout Bother, fetching the key sometimes fails, sometimes works.
    shout I will try another way, using apt-key...
    apt-key adv --recv-key --keyserver keyserver.ubuntu.com 14AA40EC0831756756D7F66C4F4EA0AAE5267A6C
    LC_ALL=C.UTF-8 add-apt-repository --yes ppa:ondrej/php
    if [ "$?" != "0" ]; then
      shout Sorry, I could not add the repository.
      shout So I cannot install PHP for you.
      pause
      exit 1
    fi
  fi
  pause
  apt update
fi

# If they are running Ubuntu 18 or later, they need more Ubuntu repos.
if [ "$OS" = "ubuntu" -a "$OSVER" -ge "18" ]; then
  shout Adding more components of Ubuntu\'s repos.
  C="$( lsb_release -sc )"
  add-apt-repository -y "deb http://archive.ubuntu.com/ubuntu ${C} restricted universe multiverse"
  add-apt-repository -y "deb http://archive.ubuntu.com/ubuntu ${C}-security restricted universe multiverse"
  add-apt-repository -y "deb http://archive.ubuntu.com/ubuntu ${C}-updates restricted universe multiverse"
  apt update
fi

# If they are running Debian 8 then need Ondrej's Jessie/Stretch APT repos
if [ "$OS" = "debian" -a "$OSVER" -lt "10" ]; then
  # Find the Debian release codename
  if canrun lsb_release; then
    CODENAME="$( lsb_release --codename 2>/dev/null | sed -e 's/^[^:]*:\s*//' )"
  else
    CODENAME="$( grep '^VERSION_CODENAME=' /etc/os-release | head -1 | sed -e 's/#.*$//; s/ *$//; s/^VERSION_CODENAME=\([a-zA-Z0-9_-]*\).*/\1/;' )"
  fi
  if [ "x$CODENAME" = "x" ]; then
    CODENAME="$( prompt "What is the codename of this Debian release" "stretch" "$CODENAME" )"
  fi
  shout Installing Ondrej Suri\'s "$CODENAME" APT repo to get PHP 7.2.
  apt -y install ca-certificates apt-transport-https gnupg
  curl https://packages.sury.org/php/apt.gpg | apt-key add -
  echo "deb https://packages.sury.org/php/ ${CODENAME} main" > /etc/apt/sources.list.d/php.list
  pause
  apt update
fi

# Find the major version of PHP they have installed
ANYPHP=yes
#PHPVER="$( dpkg-query --show --showformat='${Package}\n' 'php*-common' 2>/dev/null | sed -e 's/^php//; s/-common$//;' | sort -V | tail -1 )"
PHPVER="$( dpkg-query --show --showformat='${Package}\n' 2>/dev/null | grep '^php.*-common' | sed -e 's/^php//; s/-common$//;' | sort -V | tail -1 )"
# And what is the target version? 7.2 on 14 and above
TARGETPHP='7.2'
#if [ "x$PHPVER" = "x" ]; then
#  ANYPHP=no
#  # PHP not installed, so we're going to simulate an install first
#  # php-sodium is a troublemaker, so likely to have less compatibility.
#  # php-sodium doesn't have a version number, so pull that from phpXX-cli.
#  PHPVER="$( apt-get -s install php-sodium | \
#             grep -E '^(Inst|Conf)\s+php[0-9.]*-cli' | \
#             tail -1 | \
#             sed -e 's/^[a-zA-Z]*\s*php\([^-]*\)-.*$/\1/' )"
#fi
if [ "x$PHPVER" = "x" ]; then
  ANYPHP=no
  # PHP not installed, so we're going to simulate an install first
  # php-tidy is a random module that always ships.
  PHPVER="$( apt-get -s install php-tidy | \
             grep -E '^(Inst|Conf)\s+php[0-9.]+-tidy' | \
             tail -1 | \
             sed -e 's/^[a-zA-Z]*\s*php\([^-]*\)-.*$/\1/' )"
fi

# That may have found PHP 9 which we don't want yet.
PHPMAJORVER="$( echo $PHPVER | cut -d. -f1 )"
if (( PHPMAJORVER > 8 )); then
  ANYPHP=no
  # PHP not installed, so we're going to simulate an install first
  # php-tidy is a random module that always ships.
  # Working backwards from PHP 8.2, find the most recent version
  # of PHP 8 or 7 that is available. Min required is 7.2.
  for VER in 8.2 8.1 8.0 7.9 7.8 7.7 7.6 7.5 7.4 7.3 7.2
  do
    # In sury's repo they are called php7.2, php7.4 etc
    PHPVER="$( apt-get -s install php$VER-tidy 2>/dev/null | \
               grep -E '^(Inst|Conf)\s+php[0-9.]+-tidy' | \
               tail -1 | \
               sed -e 's/^[a-zA-Z]*\s*php\([^-]*\)-.*$/\1/' )"
    if [ "x$PHPVER" != "x" ]; then
      break
    fi
  done
fi
if [ "x$PHPVER" = "x" ]; then
  shout I cannot work out what version of PHP you will use.
  PHPVER="$( prompt "PHP version number" "$TARGETPHP" "7.2" )"
fi

# The lesser of 7.2 and their version
MINPHP="$( echo -e "$PHPVER\n$TARGETPHP" | sort -V | head -1 )"
# This is really: Is PHP installed, and is their version < $TARGETPHP ?
if [ "x$ANYPHP" = "xyes" -a "x$MINPHP" != "x$TARGETPHP" ]; then
  shout
  shout "You have a version of PHP installed which is earlier than $TARGETPHP."
  shout 'I need to totally remove that first,'
  shout 'along with packages that depend on it, as shown here:'
  PHPINSTALLED="$( dpkg-query --show --showformat='${Package}\n' 'php-*' 'php[0-9]*' '*mod-php*' )"
  # Just list the actual packages that will be removed
  apt-get --dry-run purge $PHPINSTALLED | sed -e '1,/^The following packages will be REMOVED:/d; /^[0-9]* upgraded, [0-9]* newly installed,/,$d;'
  pause
  shout
  shout 'Now I am going to do that for real!'
  shout
  pause 15
  apt-get -y purge $PHPINSTALLED
  shout 'And remove packages that are not needed any more.'
  apt-get -y autoremove
fi

MAXPHP="$( echo -e "$PHPVER\n$TARGETPHP" | sort -V | tail -1 )"
shout
shout Installing PHP version "$MAXPHP" and the modules I need.
pause
# Moved json to optional as we don't know what it's called (Ubuntu 21)
apt -y install libapache2-mod-php${MAXPHP} php${MAXPHP} php${MAXPHP}-{common,cli,sqlite3,mysql,readline,ldap,gd,curl,xml}
shout
shout I would benefit from some others but they are built-in on recent Ubuntus.
shout So do not worry if any or all of these fail.
pause
# 2022-06-01 Removed enchant from this list, I don't use it.
for MOD in intl imap mbstring sodium
do
  apt -y install php${MAXPHP}-${MOD}
done
# json could be called either of these
apt -y install php${MAXPHP}-json php-json

shout ' '
shout And enable them.
# Note use of mysqli and mysqlnd, and not just mysql when enabling PHP modules
for MOD in curl sqlite3 mysqli mysqlnd gd ldap pdo readline
do
  phpenmod -v ALL -s ALL "$MOD"
done
shout ' '
shout And these will fail to enable if they failed to install earlier.
shout So do not worry.
# 2022-06-01 Removed enchant from this list, I don't use it.
for MOD in imap intl json mbstring mysql sodium
do
  phpenmod -v ALL -s ALL "$MOD"
done

shout You should now have the PHP packages installed.

exit 0

