#!/bin/bash

# Install and setup httpd for ZendTo

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout Configure web server and PHP
shout
shout =================================================================
shout
pause

# Set up httpd
SERVERNAME="$(fullhostname)"
if [ "$OS" = "ubuntu" -a "$OSVER" -lt "14" ]; then
  HTTPDCONF=/etc/apache2/sites-available/001-zendto
  HTTPDCONFS=/etc/apache2/sites-available/001-zendto-ssl
else
  HTTPDCONF=/etc/apache2/sites-available/001-zendto.conf
  HTTPDCONFS=/etc/apache2/sites-available/001-zendto-ssl.conf
fi
if [ -f $HTTPDCONF ]; then
  shout Looks like you already have a ZendTo HTTP website
  shout configured here. I will leave it alone.
else
  shout Creating HTTP ZendTo website definition in
  shout '    '$HTTPDCONF
  shout "for ServerName $SERVERNAME"
  cat <<EOHTTPD > $HTTPDCONF
<VirtualHost *:80>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All

    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  #HTTPS# This will force http to automatically redirect to https.
  #HTTPS<IfModule mod_rewrite.c>
  #HTTPS  RewriteEngine On
  #HTTPS  RewriteCond %{HTTPS} off
  #HTTPS  RewriteCond %{REQUEST_URI} !^/\.well\-known/acme\-challenge/
  #HTTPS  RewriteRule (.*) https://%{HTTP_HOST}%{REQUEST_URI}
  #HTTPS</IfModule>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to start getting the WebDAV support working.
  # I would recommend only doing this on the https site, and
  # not on this http one unless you cannot get a https certificate.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service apache2 restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>

</VirtualHost>
EOHTTPD
fi

if [ -f $HTTPDCONFS ]; then
  shout Looks like you already have a ZendTo HTTPS website
  shout configured here. I will leave it alone.
else
  shout Creating HTTPS ZendTo website definition in
  shout '    '$HTTPDCONFS
  shout "for ServerName $SERVERNAME"
  cat <<EOHTTPDS > $HTTPDCONFS
<IfModule mod_ssl.c>

<VirtualHost *:443>
  ServerName $SERVERNAME
  DocumentRoot "/opt/zendto/www"

  # Add the "SameSite" restriction to all cookies.
  # Warning: This will break if you embed ZendTo in an iframe or similar!
  <IfModule mod_headers.c>
    Header edit Set-Cookie ^(.*)\$ \$1;SameSite=Lax
  </IfModule>

  # Rewrite anything not ending in .php so it does end in .php.
  # This allows us to totally hide the fact that ZendTo is a PHP app
  # Use this in conjunction with 'hidePHP' => TRUE in preferences.php.
  <IfModule mod_rewrite.c>
    RewriteEngine on
    RewriteCond %{REQUEST_FILENAME} !-d
    RewriteCond %{REQUEST_FILENAME}\.php -f
    RewriteRule ^(.*)\$ \$1.php
  </IfModule>

  # Need this else mod_rewrite rule above won't work due to
  # mod_negotiation.
  <IfModule mod_mime.c>
    AddType application/x-httpd-php .php
  </IfModule>

  # Enable SSL for this virtual host.
  SSLEngine on
  CustomLog \${APACHE_LOG_DIR}/zendto_access.log combined
  ErrorLog  \${APACHE_LOG_DIR}/zendto_error.log

  # Where your SSL certificate and private key live.
  SSLCertificateFile    /etc/ssl/certs/zendto-selfsigned-cert.pem
  SSLCertificateKeyFile /etc/ssl/private/zendto-selfsigned-key.pem

  # ZendTo: Not needed for a self-signed certificate like the
  #         sample one above. But when you setup your proper
  #         certificate, put the Certificate Authority's certificate
  #         chain in here, and of course uncomment it!
  #SSLCertificateChainFile /etc/ssl/certs/server-ca.crt

  # Uncomment 1 of these 2 when using client certificate authentication
  #SSLCACertificatePath /etc/ssl/certs/
  #SSLCACertificateFile /etc/apache2/ssl.crt/ca-bundle.crt

  # HSTS (mod_headers is required) (15768000 seconds = 6 months)
  Header always set Strict-Transport-Security "max-age=15768000"

  # CSP sets where different types of resource are allowed to come from
  # This is fairly strict except for images (img-src), as the service
  # logo may come from some web server of your own.
  # For further info read:
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy
  # https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Content-Security-Policy/default-src
  Header set Content-Security-Policy "default-src 'none'; script-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.gstatic.cn https://www.recaptcha.net; connect-src 'self' 'unsafe-inline'; img-src data: * ; font-src 'self' 'unsafe-inline'  https://fonts.googleapis.com https://fonts.gstatic.com; style-src 'self' 'unsafe-inline' https://fonts.googleapis.com; frame-src 'self' 'unsafe-inline' https://www.google.com https://www.gstatic.com https://www.recaptcha.net"

  # SSL Protocol Adjustments for MSIE:
  # MSIE 7 and newer should be able to use keepalive
  BrowserMatch "MSIE [17-9]" ssl-unclean-shutdown

  <FilesMatch "\.php$">
    SSLOptions +StdEnvVars
  </FilesMatch>

  <Directory "/opt/zendto/www">
    Options FollowSymLinks MultiViews
    # This controls what directives may be placed in .htaccess files
    AllowOverride All
    # Controls who can get stuff from this server file
    <IfModule !mod_authz_core.c>
      # For Apache 2.2:
      Order allow,deny
      Allow from all
    </IfModule>
    <IfModule mod_authz_core.c>
      # For Apache 2.4:
      Require all granted
    </IfModule>
  </Directory>

  # Uncomment this if you install the SAML support for ZendTo
  # and are trying to configure it.
  # Alias "/saml" "/opt/zendto/simplesamlphp/www"
  # <Directory "/opt/zendto/simplesamlphp/www">
  #   Options Indexes FollowSymLinks
  #   AllowOverride None
  #   # Controls who can get stuff from this server file
  #   <IfModule !mod_authz_core.c>
  #     # For Apache 2.2:
  #     Order allow,deny
  #     Allow from all
  #   </IfModule>
  #   <IfModule mod_authz_core.c>
  #     # For Apache 2.4:
  #     Require all granted
  #   </IfModule>
  # </Directory>

  # Uncomment this to get the WebDAV support working, so users
  # can easily upload files for the "library of files" feature.
  # You also need to run these 3 commands as root:
  #     a2enmod dav_fs
  #     a2enmod dav
  #     service apache2 restart
  #Alias /library /var/zendto/library
  #<Location /library>
  #        DAV on
  #        AuthUserFile /var/zendto/library.passwd
  #        AuthName "ZendTo Library"
  #        AuthType Basic
  #        Require valid-user
  #</Location>
</VirtualHost>

EOHTTPDS

  if [ "$OS" = "ubuntu" -a "$OSVER" -lt "14" ]; then
    cat <<EOHTTPS12 >> $HTTPDCONFS
# ZendTo: For Ubuntu 12
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
# Get version numbers with the commands "apache2 -v" and "openssl version"
SSLProtocol             all -SSLv2 -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
SSLHonorCipherOrder     on

</IfModule>
EOHTTPS12
  elif [ "$OS" = "debian" -o "$OSVER" -lt "16" ]; then
    cat <<EOHTTPS14 >> $HTTPDCONFS
# ZendTo: For Ubuntu 14 and Debian 8
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
# Get version numbers with the commands "apache2 -v" and "openssl version"
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
SSLHonorCipherOrder     on
SSLCompression          off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS14
  elif [ "$OS" = "ubuntu" -a "$OSVER" -lt "18" ]; then
    cat <<EOHTTPS16 >> $HTTPDCONFS
# ZendTo: For Ubuntu 16
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
# Get version numbers with the commands "apache2 -v" and "openssl version"
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-SHA384:ECDHE-RSA-AES256-SHA384:ECDHE-ECDSA-AES128-SHA256:ECDHE-RSA-AES128-SHA256
SSLHonorCipherOrder     on
SSLCompression          off
SSLSessionTickets       off
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS16
  else
    cat <<EOHTTPS18 >> $HTTPDCONFS
# ZendTo: For Ubuntu 18-21
# Generated by https://mozilla.github.io/server-side-tls/ssl-config-generator/
# Get version numbers with the commands "apache2 -v" and "openssl version"
SSLProtocol             all -SSLv3 -TLSv1 -TLSv1.1
SSLCipherSuite          ECDHE-ECDSA-AES128-GCM-SHA256:ECDHE-RSA-AES128-GCM-SHA256:ECDHE-ECDSA-AES256-GCM-SHA384:ECDHE-RSA-AES256-GCM-SHA384:ECDHE-ECDSA-CHACHA20-POLY1305:ECDHE-RSA-CHACHA20-POLY1305:DHE-RSA-AES128-GCM-SHA256:DHE-RSA-AES256-GCM-SHA384
SSLHonorCipherOrder     off
SSLCompression          off
SSLSessionTickets       off

SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/var/run/ocsp(128000)

</IfModule>
EOHTTPS18
  fi

  chmod u=rw,go=r $HTTPDCONF $HTTPDCONFS
  shout Generating a self-signed SSL key and certificate for you.
  #
  # First we need to modify the openssl.cnf file.
  #
  # So make a safe copy of it
  OPENSSLCNF="$( mktemp )"
  cat /etc/ssl/openssl.cnf >> "$OPENSSLCNF"
  # Find the name of the x509 extensions section
  EXTENSIONS="$( grep '^x509_extensions' "$OPENSSLCNF" | head -1 | sed -e 's/x509_extensions[^=]*=[^a-zA-Z0-9]*\([a-zA-Z0-9_-]*\).*$/\1/' )"
  # Add the 2 extension lines s to the start of the right section
  shout "Including x509 certificate extensions"
  shout "extendedKeyUsage = serverAuth"
  shout "subjectAltName = DNS:$SERVERNAME"
  sed -i -e '/^\[ *'"$EXTENSIONS"' *\]/ a \\extendedKeyUsage = serverAuth\
subjectAltName = DNS:'"$SERVERNAME" "$OPENSSLCNF"
  # Now generate the cert including the new cnf file
  ( echo GB; echo Hampshire; echo Southampton; echo ZendTo User Site; echo; \
    echo $SERVERNAME; echo ) | \
  openssl req -x509 -sha256 -nodes -days 365 -newkey rsa:2048 -extensions "$EXTENSIONS" -config "$OPENSSLCNF" -keyout /etc/ssl/private/zendto-selfsigned-key.pem -out /etc/ssl/certs/zendto-selfsigned-cert.pem >/dev/null 2>&1
  # And ditch the temporary openssl.cnf file
  rm -f "$OPENSSLCNF"

  shout Replacing the default sites with the ZendTo one
  shout and its https version.
  a2dissite 000-default >/dev/null
  a2dissite default-ssl >/dev/null
  a2ensite 001-zendto >/dev/null
  a2ensite 001-zendto-ssl >/dev/null
  shout Enabling ssl, rewrite and headers modules.
  a2enmod ssl >/dev/null
  a2enmod rewrite >/dev/null
  a2enmod headers >/dev/null
  shout
  shout 'The http and https versions of your new ZendTo site'
  shout 'will be at the addresses'
  shout '    'http://$SERVERNAME'/'
  shout and
  shout '    'https://$SERVERNAME'/'
  shout
  shout '*** Please note these down as you will need them in a minute. ***'
  shout
  shout I can configure the http site to automatically forward
  shout all connections to the https site.
  shout But because I have just used a self-signed certificate,
  shout your web browser will complain very loudly about the
  shout security of the https site the first time you visit it.
  shout
  shout 'You should normally say "yes" to this question.'
  shout 'But you might have a good reason to say "no", such as you'
  shout are just trying it out internally and are not bothered
  shout about security right now on this server.
  shout
  shout Would you like me to automatically forward http
  if yesno "connections straight to the https site" "y"; then
    sed -i 's/#HTTPS//g;' $HTTPDCONF
    shout Connections to http will go straight to https.
  fi
fi

# Select the right time zone as we need it for php.ini
shout
shout I need to know which timezone you are in.
shout
# If they are running systemd, they'll have timedatectl so use it
MYTZ="$( timedatectl status 2>/dev/null |
         grep -i 'time *zone' |
         sed -e 's/^[^:]*: *//; s/ *(.*$//;' )"
# If that failed (don't have systemd) then see if /etc/localtime points to it
if [ "x$MYTZ" = "x" -a -L /etc/localtime ]; then
  # Timezone names start with a capital letter or a number,
  # the parent directory names don't.
  MYTZ="$( readlink /etc/localtime | perl -p -e 's/^.*?\/([A-Z0-9].*$)/$1/' )"
fi
# If the timezone hasn't been set, it defaults to Etc/UTC which is nowhere
if [ "x$MYTZ" = "xEtc/UTC" ]; then
  shout Your timezone has not been set.
  shout
  MYTZ="$(tzselect | tail -1)"
fi
MYTZ="$(prompt "Time zone" "America/New_York" "$MYTZ")"
shout "Thank you, got your time zone as $MYTZ"
timedatectl set-timezone "$MYTZ"

# Setup php.ini
shout
shout Configuring PHP
pause

# Set configuration in php.ini
# Find their highest version number of PHP
PHPVER="$( dpkg-query --show --showformat='${Package}\n' 'php*-common' | sed -e 's/^php//; s/-common$//;' | sort -V | tail -1 )"
# See if they have a conf.d dir and use that if possible
if [ -d /etc/php/${PHPVER}/apache2/conf.d ]; then
  INIS=""
  for F in cli apache2 fpm
  do
    if [ -d /etc/php/${PHPVER}/${F}/conf.d ]; then
      INIS="$INIS /etc/php/${PHPVER}/$F/conf.d/99-zendto.ini"
    fi
  done
else
  INIS="$( ls -d /etc/php/${PHPVER}/{cli,apache2,fpm}/php.ini 2>/dev/null )"
fi
for F in $INIS
do
  shout "Patching $F"
  if [ -f "$F" ]; then
    cp -f "$F" "$F.zendto"
  else
    :> "$F"
  fi
  setphpini "$F" date.timezone "$MYTZ"
  setphpini "$F" max_execution_time 28800
  setphpini "$F" max_input_time 28800
  setphpini "$F" memory_limit 300M
  setphpini "$F" error_reporting 'E_ALL & ~E_DEPRECATED & ~E_STRICT & ~E_NOTICE'
  # Only do these for apache2 and fpm, not cli.
  [[ "$F" =~ '/apache2/' ]] && setphpini "$F" display_errors Off
  [[ "$F" =~ '/apache2/' ]] && setphpini "$F" display_startup_errors Off
  [[ "$F" =~ '/fpm/'     ]] && setphpini "$F" display_errors Off
  [[ "$F" =~ '/fpm/'     ]] && setphpini "$F" display_startup_errors Off
  setphpini "$F" log_errors On
  setphpini "$F" post_max_size 50000M
  setphpini "$F" file_uploads On
  setphpini "$F" upload_tmp_dir /var/zendto/incoming
  setphpini "$F" upload_max_filesize 50G
  setphpini "$F" max_file_uploads 200
  setphpini "$F" browscap /opt/zendto/lib/browscap.ini
  shout ' '
done

# Used to say < 16
if [ "$OS" = "ubuntu" -a "$OSVER" -eq "14" ]; then
  shout Removing any rogue comment lines in PHP imap.ini
  sed -i.zendto '/^#/ d' /etc/php/*/mods-available/*imap.ini 2>/dev/null
fi

shout 'Ignore warnings about "DocumentRoot does not exist".'
service apache2 restart
shout
pause

shout
shout Your web server and PHP ini files should now be
shout configured to run ZendTo on this server.
shout As mentioned above, you will need to setup your
shout own https SSL certificate and keys.
shout To find the basic settings for ZendTo, look in
shout '    '$HTTPDCONF
shout '    '$HTTPDCONFS
shout

exit 0

