#!/bin/bash

# Install ZendTo itself.

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout =================================================================
shout
shout 'Install ZendTo itself and configure email sending'
shout
shout =================================================================
shout
pause


shout Installing usage statistics graphing package, i18n tools, jq and locales
apt-get -y install rrdtool gettext jq locales

# Need nc for determining SMTP server ports
if which nc >/dev/null 2>&1; then
  shout netcat is already installed.
else
  shout Installing netcat
  apt-get -y install netcat-openbsd
fi

# Do we want to use the repo?
if [ "x$USEREPO" = "x" ]; then
  # This setting will be replaced with n if a beta installer
  USEREPO=n
fi

shout ' '
mkdir -p "$SRCSTORE" && cd "$SRCSTORE" || exit 1
REPOFAILED=no
if [ "x$USEREPO" = "xy" ]; then
  shout "Trying to use the ZendTo apt repository."
  shout Installing GPG
  apt -y install gnupg
  shout "Importing the GPG signing key."
  # Import it straight into the trusted list if possible,
  # apt-key is deprecated in modern versions.
  if [ -d /etc/apt/trusted.gpg.d ]; then
    curl -o /etc/apt/trusted.gpg.d/zendto.gpg.asc https://zend.to/files/zendto.gpg.asc
  else
    curl https://zend.to/files/zendto.gpg.asc | apt-key add -
  fi
  curl -O https://zend.to/files/zendto-repo.deb || REPOFAILED=yes
  dpkg -i zendto-repo.deb || REPOFAILED=yes
else
  # Fake repo failure, as we didn't want to use it anyway
  REPOFAILED=yes
fi

if [ "x$REPOFAILED" = "xno" ]; then
  apt-get update
  shout "Now to install ZendTo itself from the apt repository."
  shout "Drumroll please..."
  pause
  # Is it already installed? If so, upgrade rather than install
  ( if dpkg -l | awk '{ print $2 }' | grep -q '^zendto$'; then
      shout Upgrading ZendTo
      apt-get -y upgrade zendto
    else
      shout Installing ZendTo
      apt-get -y install zendto
    fi
  ) || {
    # If either of those failed...
    shout Failed to install ZendTo from the apt repository.
    shout Exiting...
    exit 1
  }
else
  shout "Failed to find ZendTo apt repository deb."
  shout "So we will find and install the latest version without it."
  VERSION="$( curl --silent https://zend.to/files/ZendTo-Version-Beta )"
  shout "According to the zend.to website, the latest version is $VERSION."
  VERSION="$( prompt "Version of ZendTo to install" "4.19-1" "$VERSION")"
  shout "Importing the GPG signing key."
  # Import it straight into the trusted list if possible,
  # apt-key is deprecated in modern versions.
  if [ -d /etc/apt/trusted.gpg.d ]; then
    curl -o /etc/apt/trusted.gpg.d/zendto.gpg.asc https://zend.to/files/zendto.gpg.asc
  else
    curl https://zend.to/files/zendto.gpg.asc | apt-key add -
  fi
  curl -O https://zend.to/files/zendto_"$VERSION".deb || {
    shout "Failed to find ZendTo deb for version ${VERSION}."
    shout "Exiting..."
    exit 1
  }
  shout "About to install ZendTo version $VERSION"
  shout "Slightly shorter and quieter drumroll please..."
  pause
  # Attempt to install it, install any outstanding deps,
  # try again to be sure.
  # Only actually care if the the last one installs correctly.
  ( dpkg -i "zendto_${VERSION}.deb";
    apt-get -y -f install;
    dpkg -i "zendto_${VERSION}.deb"
  ) || {
    shout Failed to install ZendTo deb package.
    shout Exiting...
    exit 1
  }
fi

shout And set up graphing data
php /opt/zendto/sbin/rrdUpdate.php /opt/zendto/config/preferences.php | grep -v '^[0-9]*x[0-9]*$'

if [ -x /opt/zendto/bin/makelanguages ]; then
  shout
  shout 'And update and compile translations'
  /opt/zendto/bin/makelanguages
fi

# Set up the SMTP settings in preferences.php
configurePHPMailer
# Set up the internaldomains.conf file
configureInternalDomains
# Set up their zendto.conf file
configureZendToConf

# If we are using mod_rewrite to hide PHP extensions, tell ZendTo
if grep -q '^ *RewriteRule.*1\.php' /etc/apache2/sites-available/001-zendto-ssl.conf 2>/dev/null; then
  shout 'Enabled PHP hiding'
  setPrefphp hidePHP notquoted "TRUE"
fi

# Set the cookieSecret in preferences.php
shout "Setting 'cookieSecret' in ZendTo preferences.php"
shout "If this takes a long time, it needs more entropy (randomness)."
shout "So wiggle your mouse around, login separately to the ZendTo server"
shout "and run commands, type stuff, make it do almost anything. It will"
shout "get there eventually."
COOKIESECRET="$( /opt/zendto/sbin/genCookieSecret.php | grep "'cookieSecret' *=" | awk '{ print $NF }' | tr -d "'" )"
setPrefphp cookieSecret quoted "$COOKIESECRET"

# Set the serverRoot setting in preferences.php
# First find the site config file, then work out if we're sure that
# we wrote it, and the RewriteRule is not commented out.
# If so, change the http to https in 'serverRoot'.
SERVERNAME="$(fullhostname)"
if [ "$OS" = "ubuntu" -a "$OSVER" -lt "14" ]; then
  HTTPDCONF=/etc/apache2/sites-available/000-zendto
else
  HTTPDCONF=/etc/apache2/sites-available/001-zendto.conf
fi
if [ -f "$HTTPDCONF" ]; then
  # The site is there
  if grep -q '^ *RewriteRule.*https:' "$HTTPDCONF"; then
    # The RewriteRule is in place, so we're forwarding to https
    SERVERROOT="https://$SERVERNAME/"
    DEFROOT="https://zendto.soton.ac.uk/"
  else
    SERVERROOT="http://$SERVERNAME/"
    DEFROOT="http://zendto.soton.ac.uk/"
  fi
  shout
  shout "The address of your new ZendTo site MUST end with a '/'"
  SERVERROOT="$( prompt "What is the address of your new ZendTo site" "$DEFROOT" "$SERVERROOT")"
  setPrefphp serverRoot quoted "$SERVERROOT"
  shout "I have set your 'serverRoot' value in preferences.php to"
  shout "  'serverRoot' => '$SERVERROOT'"
  shout 'Be sure to check this is correct, or ZendTo will fail in'
  shout 'strange ways.'
  pause 10
fi

shout
shout
shout 'Please ensure your /opt/zendto/config/* files are up to date.'
shout
shout 'To help you, there is a tool for upgrading the preferences.php'
shout 'and zendto.conf files.'
shout 'Simply run'
shout '    /opt/zendto/bin/upgrade'
shout
shout 'To edit any text in the web interface, or add/edit a language,'
shout 'see'
shout '    https://zend.to/translators'
shout
pause 20

shout
shout ZendTo itself has been installed.
shout

exit 0
